#include "ServerProperties.h"
#include <iostream>
#include <fstream>
#include <string_view>
#include <algorithm>
#include <ctime>
#include <cstring>
#include <iomanip>

#include "../util/StringUtil.h"

namespace Feather
{
    void ServerProperties::Load()
    {
        std::ifstream file;
        file.open(path);

        printf("Reading %s...\n", path);
        if (file.is_open())
        {
            string line;
            while(std::getline(file, line))
            {
                // trip whitespace
                StringUtil::trim(line);

                // skip comments
                if (line[0] == '#') continue;
                
                // key=value
                size_t pos = line.find('=');

                // java also accepts key:value
                if (pos == string::npos)
                    pos = line.find(':');

                if (pos == string::npos)
                    continue;
                
                string key = line.substr(0, pos);
                StringUtil::trim(key);
                string value = line.substr(pos + 1);
                StringUtil::trim(value);

                properties.insert( {key, value} );
            }
            file.close();
        }
        else
        {
            printf("Could not find %s\n", path);
        }
    }

    void ServerProperties::Save()
    {
        std::ofstream file;
        file.open(path);

        file << "#Minecraft server properties\n";

        auto time = std::time(nullptr);
        auto localTime = *std::localtime(&time);

        // The following code abbreviates a time zone like "Pacifc Daylight Time" 
        // to a short form like "PDT". If the time zone is already abbreviated
        // then it won't change the abbreviation

        char timeZone[256];
        strftime(timeZone, 256, "%Z", &localTime);

        char timeZoneAbbr[5];
        int counter = 0;
        for (int i = 0; i < strlen(timeZone); i++)
        {
            char c = timeZone[i];
            if (std::isupper(c)) {
                timeZoneAbbr[counter++] = c;
                // stop before last char to insert 0
                if (counter >= 4) {
                    timeZoneAbbr[counter] = 0;
                    break;
                }
            }
        }

        // Ex: Thu Jul 23 01:43:47 PDT 2020
        file << "#" << std::put_time(&localTime, "%a %b %d %H:%M:%S ") << timeZoneAbbr << std::put_time(&localTime, " %Y") << "\n";

        file << onlineMode.key << "=" << onlineMode.ToString() << "\n";
        file << preventProxyConnections.key << "=" << preventProxyConnections.ToString() << "\n";
        file << serverIp.key << "=" << serverIp.ToString() << "\n";
        file << spawnAnimals.key << "=" << spawnAnimals.ToString() << "\n";
        file << spawnNpcs.key << "=" << spawnNpcs.ToString() << "\n";
        file << pvp.key << "=" << pvp.ToString() << "\n";
        file << allowFlight.key << "=" << allowFlight.ToString() << "\n";
        file << resourcePack.key << "=" << resourcePack.ToString() << "\n";
        file << motd.key << "=" << motd.ToString() << "\n";
        file << forceGameMode.key << "=" << forceGameMode.ToString() << "\n";
        file << enforceWhitelist.key << "=" << enforceWhitelist.ToString() << "\n";
        file << levelName.key << "=" << levelName.ToString() << "\n";
        file << serverPort.key << "=" << serverPort.ToString() << "\n";
        file << maxBuildHeight.key << "=" << maxBuildHeight.ToString() << "\n";
        //file << announcePlayerAchievements.key << "=" << announcePlayerAchievements.ToString() << "\n";
        file << enableQuery.key << "=" << enableQuery.ToString() << "\n";
        file << queryPort.key << "=" << queryPort.ToString() << "\n";
        file << enableRcon.key << "=" << enableRcon.ToString() << "\n";
        file << rconPort.key << "=" << rconPort.ToString() << "\n";
        file << rconPassword.key << "=" << rconPassword.ToString() << "\n";
        //file << resourcePackHash.key << "=" << resourcePackHash.ToString() << "\n";
        file << resourcePackSha1.key << "=" << resourcePackSha1.ToString() << "\n";
        file << hardcore.key << "=" << hardcore.ToString() << "\n";
        file << allowNether.key << "=" << allowNether.ToString() << "\n";
        file << spawnMonsters.key << "=" << spawnMonsters.ToString() << "\n";
        file << snooperEnabled.key << "=" << snooperEnabled.ToString() << "\n";
        file << useNativeTransport.key << "=" << useNativeTransport.ToString() << "\n";
        file << enableCommandBlock.key << "=" << enableCommandBlock.ToString() << "\n";
        file << spawnProtection.key << "=" << spawnProtection.ToString() << "\n";
        file << opPermissionLevel.key << "=" << opPermissionLevel.ToString() << "\n";
        file << functionPermissionLevel.key << "=" << functionPermissionLevel.ToString() << "\n";
        file << maxTickTime.key << "=" << maxTickTime.ToString() << "\n";
        file << viewDistance.key << "=" << viewDistance.ToString() << "\n";
        file << maxPlayers.key << "=" << maxPlayers.ToString() << "\n";
        file << networkCompressionThreshold.key << "=" << networkCompressionThreshold.ToString() << "\n";
        file << broadcastRconToOps.key << "=" << broadcastRconToOps.ToString() << "\n";
        file << broadcastConsoleToOps.key << "=" << broadcastConsoleToOps.ToString() << "\n";
        file << maxWorldSize.key << "=" << maxWorldSize.ToString() << "\n";
        file << syncChunkWrites.key << "=" << syncChunkWrites.ToString() << "\n";
        file << enableJmxMonitoring.key << "=" << enableJmxMonitoring.ToString() << "\n";
        file << enableStatus.key << "=" << enableStatus.ToString() << "\n";
        file << entityBroadcastRangePercentage.key << "=" << entityBroadcastRangePercentage.ToString() << "\n";
        file << playerIdleTimeout.key << "=" << playerIdleTimeout.ToString() << "\n";
        file << whiteList.key << "=" << whiteList.ToString() << "\n";

        file << "\n";
        file.close();
    }
}